package com.minecraftserverzone.weaponmaster.itemlayers;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;

import com.minecraftserverzone.weaponmaster.WeaponMasterMod;
import com.minecraftserverzone.weaponmaster.renderer.quiver.QuiverWithArrowsModel;
import com.minecraftserverzone.weaponmaster.setup.configs.ConfigHelper;
import com.minecraftserverzone.weaponmaster.setup.events_on_client.ClientOnlyForgeSetup;
import com.minecraftserverzone.weaponmaster.setup.helper.ModUtils;
import com.minecraftserverzone.weaponmaster.setup.helper.ServerHelper;

import net.minecraft.client.Minecraft;
import net.minecraft.client.entity.EntityPlayerSP;
import net.minecraft.client.model.ModelBiped;
import net.minecraft.client.model.ModelRenderer;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.block.model.ItemCameraTransforms.TransformType;
import net.minecraft.client.renderer.entity.RenderLivingBase;
import net.minecraft.client.renderer.entity.layers.LayerRenderer;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemArmor;
import net.minecraft.item.ItemArrow;
import net.minecraft.item.ItemBanner;
import net.minecraft.item.ItemBow;
import net.minecraft.item.ItemShield;
import net.minecraft.item.ItemSpectralArrow;
import net.minecraft.item.ItemStack;
import net.minecraft.item.ItemSword;
import net.minecraft.item.ItemTool;
import net.minecraft.util.EnumHandSide;
import net.minecraft.util.ResourceLocation;

public class HumanoidItemLayer implements LayerRenderer<EntityLivingBase> {
	public QuiverWithArrowsModel<EntityPlayer> quiver;
	public QuiverWithArrowsModel<EntityPlayer> arrows;
	public static ResourceLocation QUIVER = new ResourceLocation(WeaponMasterMod.MODID, "textures/entity/quiver.png");
	public static ResourceLocation ARROW_TEXTURE = new ResourceLocation(WeaponMasterMod.MODID, "textures/entity/normal_arrow.png");
	public static ResourceLocation ARROW_TEXTURE_SPECTRAL = new ResourceLocation(WeaponMasterMod.MODID, "textures/entity/spectral_arrow.png");
	public final RenderLivingBase<?> modelrenderer;
	
	public HumanoidItemLayer(RenderLivingBase<?> p_174516_) {
		modelrenderer = p_174516_;
	}

	public void renderPlayerWithItems(EntityPlayer p_174525_) {
	   if(quiver == null) {
		   quiver = new QuiverWithArrowsModel(64, 64);
	   }
	   if(arrows == null) {
		   arrows = new QuiverWithArrowsModel(64, 64);
	   }
	   if(p_174525_ instanceof EntityPlayerSP) {
		   this.renderWithItems(p_174525_);
	   }else 
		   if(WeaponMasterMod.isMultiplayer && p_174525_ instanceof EntityPlayer && modelrenderer.func_177087_b() instanceof ModelBiped) {
		   RenderHelper.renderOtherPlayerWithItems(modelrenderer, (EntityPlayer) p_174525_, (ModelBiped) modelrenderer.func_177087_b());
	   }
	}

	//clientOnly functions
	public boolean blackListed(ItemStack stack){
		if(stack != null) {
			Item item = stack.func_77973_b();
			if(ClientOnlyForgeSetup.blacklist != null){
				String[] blacklist = ClientOnlyForgeSetup.blacklist.split(",");
				for(String bl : blacklist){
					String itemname = "";
					if(item != null && ModUtils.stringExist(item.getRegistryName().func_110623_a())) {
						itemname = item.getRegistryName().func_110623_a();
						
						if(ModUtils.stringContains(itemname, ".")) {
							itemname = StringUtils.substringAfterLast(itemname, ".");
						}
					}

					if(ModUtils.stringEquals(itemname, bl)){
						return true;
					}
				}
			}
		}
			
		return false;
	}

	public boolean whiteListed(ItemStack stack){
		if(ClientOnlyForgeSetup.whitelist != null && stack != null){
			Item item = stack.func_77973_b();
			
			String[] whitelist = ClientOnlyForgeSetup.whitelist.split(",");
			for(String bl : whitelist){
				String itemname = "";
				if(item != null && ModUtils.stringExist(item.getRegistryName().func_110623_a())) {
					itemname = item.getRegistryName().func_110623_a();
					
					if(ModUtils.stringContains(itemname, ".")) {
						itemname = StringUtils.substringAfterLast(itemname, ".");
					}
				}
				
				if(ModUtils.stringEquals(itemname, bl)){
					return true;
				}
			}
		}
		return false;
	}
	
	private int hasItems(EntityPlayer player, String[][] moveTypes, int slot, int selected, int shieldSlot, int bannerSlot, int[] toggleSlots) {
		   int i = 0;
		   for(int j = 0; j < 9; j++) {
			   if(moveTypes[slot][0].contains(String.valueOf(j+1))) {
				   if(!ServerHelper.isEmpty(player.field_71071_by.func_70301_a(j)) && player.field_71071_by.field_70461_c != j && toggleSlots[j] == 1) { // not empty and not selected
					   i++;
				   }
			   }
		   }
		   
		   if(moveTypes[slot][0].contains("shield")) {
			   if(shieldSlot > -1 && player.field_71071_by.field_70461_c != shieldSlot && toggleSlots[9] == 1) { // not empty and not selected
				   i++;
			   }
		   }
		   if(moveTypes[slot][0].contains("banner")) {

			   if(bannerSlot > -1 && player.field_71071_by.field_70461_c != bannerSlot && toggleSlots[10] == 1) { // not empty and not selected
				   i++;

			   }
		   }
		   
		   if(moveTypes[slot][0].contains("quiver")) {
			   for(int j = 0; j < 9; j++) {
				   if(ServerHelper.getProjectile(player, player.field_71071_by.func_70301_a(j)) != null) {
					   if(ServerHelper.getProjectile(player, player.field_71071_by.func_70301_a(j)).func_77973_b() instanceof ItemArrow) {
						   if(!ServerHelper.isEmpty(player.field_71071_by.func_70301_a(j)) && toggleSlots[10] == 1) { // not empty and not selected
							   i++;
							   break;
						   }
					   }
				   }
			   }
		   }
		   
		   return i;
	   }
	   
	//end of client only functions
	@Override
	public void func_177141_a(EntityLivingBase p_117207_, float limbSwing, float limbSwingAmount, float partialTicks, float ageInTicks, float netHeadYaw, float headPitch, float scaleIn) {
			 GlStateManager.func_179094_E();
			 
			 if (p_117207_.func_70093_af()) {
		            GlStateManager.func_179109_b(0.0F, 0.2F, 0.0F);
		  	 }
//	         if(ModList.isLoaded("yearsc")) {
//	        	 if (modelrenderer.getMainModel().isChild) {
//	 	            float f = 0.5F;
//	 	            GlStateManager.translate(0.0D, 0.75D, 0.0D);
//	 	            GlStateManager.scale(f, f, f);
//	 	         } 
//	         }

	         this.renderPlayerWithItems((EntityPlayer) p_117207_);
	         GlStateManager.func_179121_F();;
	}

   private void move(String[][] moveTypes, int slot, float translation) {
	   if(moveTypes[slot][2].equals("-")) {
		   translation = -translation;
	   }
	   if(moveTypes[slot][1].equals("x")) {
		   GlStateManager.func_179109_b(translation, 0, 0);
	   }else if(moveTypes[slot][1].equals("y")) {
		   GlStateManager.func_179109_b(0, translation, 0);
	   }else if(moveTypes[slot][1].equals("z")) {
		   GlStateManager.func_179109_b(0, 0, translation);
	   }
   }

   
   private boolean hasEquipments(EntityPlayer player, String[][] moveTypes, int slot) {
	   boolean move = false;
//	   FEET(EquipmentSlot.Type.ARMOR, 0, 1, "feet"),
//	   LEGS(EquipmentSlot.Type.ARMOR, 1, 2, "legs"),
//	   CHEST(EquipmentSlot.Type.ARMOR, 2, 3, "chest"),
//	   HEAD(EquipmentSlot.Type.ARMOR, 3, 4, "head");
	   
	   if(moveTypes[slot][0].contains("helm")) {
		   if(player.field_71071_by.func_70440_f(3) != null) {
			   move = true;
		   }
	   }
	   if(moveTypes[slot][0].contains("chest")) {
		   if(player.field_71071_by.func_70440_f(2) != null) {
			   move = true;
		   }
	   }
	   if(moveTypes[slot][0].contains("legs")) {
		   if(player.field_71071_by.func_70440_f(1) != null) {
			   move = true;
		   }
	   }
	   if(moveTypes[slot][0].contains("boots")) {
		   if(player.field_71071_by.func_70440_f(0) != null) {
			   move = true;
		   }
	   }
	   
	   return move;
   }

   private void renderWithItems(EntityPlayer p_174518_) {
	   String[] moveSlots = ClientOnlyForgeSetup.slotMover.split("_"); //divide to 12 slots
	   String[][] moveTypes = {
					moveSlots[0].split(";"),
					moveSlots[1].split(";"),
					moveSlots[2].split(";"),
					moveSlots[3].split(";"),
					moveSlots[4].split(";"),
					moveSlots[5].split(";"),
					moveSlots[6].split(";"),
					moveSlots[7].split(";"),
					moveSlots[8].split(";"),
					moveSlots[9].split(";"),
					moveSlots[10].split(";"),
					moveSlots[11].split(";"),
					moveSlots[12].split(";")
				};

	   EntityPlayer p = (EntityPlayer) p_174518_; 
		   int selected = p.field_71071_by.field_70461_c;
		   int n = 0;
		   ItemStack stack1 = blackListed(p.field_71071_by.func_70301_a(n)) ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n) == null ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n);n++;
		   ItemStack stack2 = blackListed(p.field_71071_by.func_70301_a(n)) ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n) == null ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n);n++;
		   ItemStack stack3 = blackListed(p.field_71071_by.func_70301_a(n)) ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n) == null ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n);n++;
		   ItemStack stack4 = blackListed(p.field_71071_by.func_70301_a(n)) ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n) == null ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n);n++;
		   ItemStack stack5 = blackListed(p.field_71071_by.func_70301_a(n)) ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n) == null ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n);n++;
		   ItemStack stack6 = blackListed(p.field_71071_by.func_70301_a(n)) ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n) == null ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n);n++;
		   ItemStack stack7 = blackListed(p.field_71071_by.func_70301_a(n)) ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n) == null ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n);n++;
		   ItemStack stack8 = blackListed(p.field_71071_by.func_70301_a(n)) ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n) == null ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n);n++;
		   ItemStack stack9 = blackListed(p.field_71071_by.func_70301_a(n)) ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n) == null ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n);n++;n++;
		   ItemStack ammo = blackListed(p.field_71071_by.func_70301_a(n)) ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n) == null ? ServerHelper.AIR_STACK : p.field_71071_by.func_70301_a(n);
		   ItemStack offhand = blackListed(p.func_184592_cb()) ? ServerHelper.AIR_STACK : p.func_184592_cb() == null ? ServerHelper.AIR_STACK : p.func_184592_cb();
		   ItemStack[] items = { stack1, stack2, stack3, stack4, stack5, stack6, stack7, stack8, stack9, offhand, ammo};
		   
		   int[] toggleSlots = {1,1,1,1,1,1,1,1,1,1,1,1,1,1};
		   int[] slotPositions = new int[WeaponMasterMod.maxDisplaySlotNum * 3];
		   int[] slotRotations = new int[WeaponMasterMod.maxDisplaySlotNum * 3];
		   int[] scale = new int[WeaponMasterMod.maxDisplaySlotNum];
		   
		   //attachments
		   String[] bodyattachments = ClientOnlyForgeSetup.slotAttachment.split(",");
		   String[][] bodyattachment = new String[WeaponMasterMod.maxDisplaySlotNum][2];
		   
		   for(int i = 0; i < WeaponMasterMod.maxDisplaySlotNum; i++) {
			   bodyattachment[i][0] = bodyattachments[i].split("-")[0];
			   bodyattachment[i][1] = bodyattachments[i].split("-")[1];
		   }
		   
		   if(p instanceof EntityPlayerSP) {
			   int[] toggleSlots2 = {
						ConfigHelper.TOGGLE_SLOT_MODEL[0] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[1] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[2] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[3] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[4] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[5] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[6] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[7] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[8] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[9] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[10] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[11] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[12] ? 1 : 0,
						ConfigHelper.TOGGLE_SLOT_MODEL[13] ? 1 : 0
						
				};
			   toggleSlots = toggleSlots2;

			   slotPositions = ClientOnlyForgeSetup.slotPositions;
			   slotRotations = ClientOnlyForgeSetup.slotRotations;
			   scale = ClientOnlyForgeSetup.scale;
		   }
		     
		   if((ClientOnlyForgeSetup.lastMainhandHit >= ClientOnlyForgeSetup.hideTick && ConfigHelper.TOGGLE_SLOT_MODEL[11])) {
		    	  selected = -1;
		   }
		   
		  int shieldAt = -1;
	      shieldSearchLoop: {
	    	  for(int i = 0; i < 9; i++) {
	    		  if(items[i] != null) {
	    			  if(items[i].func_77973_b() instanceof ItemShield) {
		    			  shieldAt = i;
		    			  
		    			  break shieldSearchLoop;
		    		  }
	    		  }
	    	  }
	      }

	      int bannerAt = -1;
	      bannerSearchLoop: {
	    	  for(int i = 0; i < 9; i++) {
	    		  if(items[i] != null) {
	    			  if(items[i].func_77973_b() instanceof ItemBanner) {
		    			  bannerAt = i;
		    			  break bannerSearchLoop;
		    		  }
	    		  }
	    	  }
	      }
	     //render banner if not in main hand
	      if(bannerAt != -1 && selected != bannerAt && toggleSlots[10] == 1){
	    	  GlStateManager.func_179094_E();
	    	  copyModelPartRotation(10, bodyattachment, 0.01f, 1f);
	    	  
	    	  float moveBy2 = hasItems(p, moveTypes, 10, selected, shieldAt, bannerAt, toggleSlots) * 0.042f;
		      move(moveTypes, 10, moveBy2);
	    	  float moveBy = hasEquipments(p, moveTypes, 10) ? 0.038f : 0;
	    	  move(moveTypes, 10, moveBy);
	    	  
	    	  int[] add = uniqueSettings(items[bannerAt], 10);
	    	  
	    	  GlStateManager.func_179109_b(((float)(slotPositions[30] + add[0])/100f), ((float)(slotPositions[31] + add[1])/100f), ((float)(slotPositions[32] + add[2])/100f));
	    	  GlStateManager.func_179114_b(slotRotations[30] + add[3], 1, 0, 0);
		      GlStateManager.func_179114_b(slotRotations[31] + add[4], 0, 1, 0);
		      GlStateManager.func_179114_b(slotRotations[32] + add[5], 0, 0, 1);
	    	  
		      GlStateManager.func_179114_b(180.0F, 1, 0, 0);
		      GlStateManager.func_179137_b(0.0F, 0.6D, -0.3D);
	    	  float scalePlus = (scale[10]+0.001f + add[6])/100;
	    	  float f = 0.75f;
//	          boolean flag = handSide == EnumHandSide.LEFT;
	    	  GlStateManager.func_179152_a(f + scalePlus, f + scalePlus, f + scalePlus);
	          Minecraft.func_71410_x().func_175597_ag().func_187462_a(p_174518_, items[bannerAt], TransformType.NONE, false);
//	          Minecraft.getMinecraft().getItemRenderer().renderItemSide(p_174518_, items[bannerAt], TransformType.NONE, false, p_174522_, p_174523_);
	          GlStateManager.func_179121_F();
	      }
	      GlStateManager.func_179094_E();
		  
	      //render shield if not in main hand
	      if(shieldAt != -1 && selected != shieldAt && toggleSlots[9] == 1){
	    	  GlStateManager.func_179094_E();
	    	  copyModelPartRotation(9, bodyattachment, 1f, 1f);
	    	  
	    	  float moveBy2 = hasItems(p, moveTypes, 9, selected, shieldAt, bannerAt, toggleSlots) * 0.042f;
		      move(moveTypes, 9, moveBy2);
	    	  float moveBy = hasEquipments(p, moveTypes, 9)? 0.038f : 0;
	    	  move(moveTypes, 9, moveBy);
	    	  
	    	  int[] add = uniqueSettings(items[shieldAt], 9);
	    	  
	    	  GlStateManager.func_179109_b(((float)(slotPositions[27] + add[0])/100f), ((float)(slotPositions[28] + add[1])/100f), ((float)(slotPositions[29] + add[2])/100f));
	    	  
	    	  GlStateManager.func_179137_b(0.0F, 0.0D, 0.015D);
	          
	    	  GlStateManager.func_179114_b(slotRotations[27] + add[3], 1, 0, 0);
	    	  GlStateManager.func_179114_b(slotRotations[28] + add[4], 0, 1, 0);
	    	  GlStateManager.func_179114_b(slotRotations[29] + add[5], 0, 0, 1);
		      
	    	  GlStateManager.func_179137_b(0.3F, 0.7D, 0.5D);

	    	  float scalePlus = (scale[9]+0.001f + add[6])/100f;
	    	  float f = 0.6f;
	    	  GlStateManager.func_179152_a(f + scalePlus, f + scalePlus, f + scalePlus);
	          
	    	  GlStateManager.func_179137_b(-0.25F, -0.375D, -0.625D);
	    	  GlStateManager.func_179114_b(-90, 0, 1, 0);
	          Minecraft.func_71410_x().func_175597_ag().func_187462_a(p_174518_, items[shieldAt], TransformType.THIRD_PERSON_RIGHT_HAND, false);
	          GlStateManager.func_179121_F();
	      }
	      
	      
	      
	      
	      
//////////quiver TODO //////////////////////////////////////////////////////////////////////////////////////////////////////
	      if(p.field_71071_by != null && toggleSlots[13] == 1) {
	    	  boolean hasBowOrCrossbow = false;
	    	  ItemStack projectile = ServerHelper.AIR_STACK;
	    	  
	    	  for(int i = 0; i < 9; i++) {
	    		  if(items[i] != null) {
	    			  if(items[i].func_77973_b() instanceof ItemBow) {
		    			  projectile = ServerHelper.getProjectile(p, items[i]);
		    			  hasBowOrCrossbow = true;
		    			  break;
		    		  }
	    		  }
	    	  }
	    	  
	    	  if(ServerHelper.itemStackExist(projectile)) {
	    		  int size = projectile.field_77994_a;
		    	  
		    	  if(size > 0 && hasBowOrCrossbow) {
		    		  GlStateManager.func_179094_E();
		    		  copyModelPartRotation(12, bodyattachment, 1f, 1f);
		    		  int[] add = {0, 0, 0, 0, 0, 0, 0}; //uniqueSettings(items[shieldAt], 9);
		    		  GlStateManager.func_179109_b(((float)(slotPositions[36] + add[0])/100f), ((float)(slotPositions[37] + add[1])/100f), ((float)(slotPositions[38] + add[2])/100f));

		    		  float moveBy = hasItems(p, moveTypes, 12, selected, shieldAt, bannerAt, toggleSlots) * 0.042f;
				      move(moveTypes, 12, moveBy);
				      float moveByArmor = hasEquipments(p, moveTypes, 12) ? 0.038f : 0;
			    	  move(moveTypes, 12, moveByArmor);
		    		  
		    		  
		    		  GlStateManager.func_179114_b(slotRotations[36] + add[3], 1, 0, 0);
		    		  GlStateManager.func_179114_b(slotRotations[37] + add[4], 0, 1, 0);
		    		  GlStateManager.func_179114_b(slotRotations[38] + add[5], 0, 0, 1);
		    		  float scalePlus = (scale[12]+0.001f + add[6])/200f;
			    	  float f = 0.5f;
			    	  GlStateManager.func_179152_a(f + scalePlus, f + scalePlus, f + scalePlus);
			          
		    		  
			          this.quiver.setupAnim(0, (EntityPlayer) p, 0, 0, 0, 0, 0);
		    		  ResourceLocation texture = QUIVER;
		    		  
		    		  if(projectile.func_77973_b() != null && projectile.func_77973_b() != ServerHelper.AIR) {
		    		  	  if(projectile.func_77973_b() instanceof ItemSpectralArrow) {
			    			  ResourceLocation arrow = ARROW_TEXTURE_SPECTRAL;
			    			  modelrenderer.func_110776_a(arrow);
//				    		  IVertexBuilder vertexConsumer = p_174522_.getBuffer(RenderType.getEntityCutoutNoCull(arrow));
				    		  this.arrows.setupAnim(size, (EntityPlayer) p, 0, 0, 0, 0, 0);
				    		  this.arrows.func_78088_a((EntityPlayer) p, 0, 0, 0, 0, 0, 1);
				    		  
			    		  }else if(projectile.func_77973_b() instanceof ItemArrow) {
			    			  ResourceLocation arrow = ARROW_TEXTURE;
			    			  modelrenderer.func_110776_a(arrow);
//				    		  IVertexBuilder vertexConsumer = p_174522_.getBuffer(RenderType.getEntityCutoutNoCull(arrow));
				    		  this.arrows.setupAnim(size, (EntityPlayer) p, 0, 0, 0, 0, 0);
				    		  this.arrows.func_78088_a((EntityPlayer) p, 0, 0, 0, 0, 0, 1);
			    		  }
		    		  }
		    		  
		    		  modelrenderer.func_110776_a(texture);
//		    		  IVertexBuilder vertexConsumer = p_174522_.getBuffer(RenderType.getEntityCutoutNoCull(texture));
		    		  this.quiver.func_78088_a((EntityPlayer) p, 0, 0, 0, 0, 0, 1);
		    		  
		    		  GlStateManager.func_179121_F();
		    	  }
	    	  }
	      }
///////////////////////end of quiver///////////////////////////////////////////////
	      
	      
	      
	      //0
	      for(int slot = 0; slot < 9; slot++) {
		      if(!ServerHelper.isEmpty(items[slot]) && selected != slot && shieldAt != slot && bannerAt != slot && toggleSlots[slot] == 1) {
		    	  GlStateManager.func_179094_E();
		    	  copyModelPartRotation(slot, bodyattachment, 1f, 1f);

		    	  int[] add = uniqueSettings(items[slot], slot);
		    	  
		    	  GlStateManager.func_179109_b(((float)(slotPositions[slot*3] + add[0])/100f), ((float)(slotPositions[(slot*3)+1] + add[1])/100f), ((float)(slotPositions[(slot*3)+2] + add[2])/100f));
		          float moveBy = hasItems(p, moveTypes, slot, selected, shieldAt, bannerAt, toggleSlots) * 0.042f;
			      move(moveTypes, slot, moveBy);
			      float moveByArmor = hasEquipments(p, moveTypes, slot) ? 0.038f : 0;
		    	  move(moveTypes, slot, moveByArmor);
	
		    	  
		    	  GlStateManager.func_179114_b(slotRotations[(slot * 3)] + add[3], 1, 0, 0);
		    	  GlStateManager.func_179114_b(slotRotations[(slot * 3) + 1] + add[4], 0, 1, 0);
		    	  GlStateManager.func_179114_b(slotRotations[(slot * 3) + 2] + add[5], 0, 0, 1);
	
			      TransformType ctx = TransformType.THIRD_PERSON_RIGHT_HAND;
			      boolean rightHanded = p_174518_.func_184591_cq() == EnumHandSide.RIGHT;
			      if(slot == 11) {
			    	  ctx = rightHanded ? TransformType.THIRD_PERSON_RIGHT_HAND : TransformType.THIRD_PERSON_LEFT_HAND;
			      }
//			      else if(slot == 12) {
//			    	  ctx = !rightHanded ? TransformType.THIRD_PERSON_RIGHT_HAND : TransformType.THIRD_PERSON_LEFT_HAND;
//			      }
			      
			      float scalePlus = (scale[slot]+0.001f + add[6])/100f;
		    	  float f = 1f;
		    	  GlStateManager.func_179152_a(f + scalePlus, f + scalePlus, f + scalePlus);

		    	  if(items[slot].func_77973_b() instanceof ItemBow) {
		    		  renderBowItem(slot, p_174518_, items[slot], ctx);
		    	  }else if((items[slot].func_77973_b() instanceof ItemTool || items[slot].func_77973_b() instanceof ItemSword) && !(items[slot].func_77973_b() instanceof ItemArmor)) {
		    		  renderTieredItem(slot, p_174518_, items[slot], ctx);
		      	  }else if(whiteListed(items[slot])) {
		      		  GlStateManager.func_179137_b(0.0F, 0.7D, 0.0D);
		      		  GlStateManager.func_179114_b(180, 0, 0, 1);
		    		  renderNormalItem(slot, p_174518_, items[slot], ctx);
		    	  }
		    	  GlStateManager.func_179121_F();
		      	}
   		  	}
	      
	      //offhand
	      int itemslot = 9;
	      int slot = 11;

	      if(!ServerHelper.isEmpty(items[itemslot]) && toggleSlots[slot + 1] == 1 && ClientOnlyForgeSetup.lastOffhandHit >= ClientOnlyForgeSetup.hideTick) {
	    	  if(items[itemslot].func_77973_b() != null && items[itemslot].func_77973_b() instanceof ItemShield) {
	    		  GlStateManager.func_179094_E();
		    	  copyModelPartRotation(slot, bodyattachment, 1f, 1f);
		    	  int[] add = uniqueSettings(items[itemslot], 11);

		    	  GlStateManager.func_179109_b(((float)(slotPositions[slot*3] + add[0])/100f), ((float)(slotPositions[(slot*3)+1] + add[1])/100f), ((float)(slotPositions[(slot*3)+2] + add[2])/100f));
		          float moveBy = hasItems(p, moveTypes, slot, selected, shieldAt, bannerAt, toggleSlots) * 0.042f;
			      move(moveTypes, slot, moveBy);
			      float moveByArmor = hasEquipments(p, moveTypes, slot) ? 0.038f : 0;
		    	  move(moveTypes, slot, moveByArmor);

		    	  GlStateManager.func_179114_b(slotRotations[(slot * 3)] + add[3], 1, 0, 0);
		    	  GlStateManager.func_179114_b(slotRotations[(slot * 3) + 1] + add[4], 0, 1, 0);
		    	  GlStateManager.func_179114_b(slotRotations[(slot * 3) + 2] + add[5], 0, 0, 1);
			      
			      TransformType ctx = TransformType.THIRD_PERSON_RIGHT_HAND;
			      			      
			      GlStateManager.func_179137_b(0.3F, 0.35D, 0.38D);

		    	  float scalePlus = (scale[slot]+0.001f + add[6])/100f;
		    	  float f = 0.6f;
		          GlStateManager.func_179152_a(f + scalePlus, f + scalePlus, f + scalePlus);
		          
		          GlStateManager.func_179137_b(-0.25F, -0.375D, -0.625D);
		          GlStateManager.func_179114_b(-90, 0, 1, 0);
		          GlStateManager.func_179114_b(-35, 1, 0, 0);
		          Minecraft.func_71410_x().func_175597_ag().func_187462_a(p_174518_, items[itemslot], ctx, false);
		          GlStateManager.func_179121_F();
	    		  
	    	  }else {
	    		  GlStateManager.func_179094_E();
		    	  copyModelPartRotation(slot, bodyattachment, 1f, 1f);
		    	  int[] add = uniqueSettings(items[itemslot], 11);
		    	  
		    	  GlStateManager.func_179109_b(((float)(slotPositions[slot*3] + add[0])/100f), ((float)(slotPositions[(slot*3)+1] + add[1])/100f), ((float)(slotPositions[(slot*3)+2] + add[2])/100f));
		          float moveBy = hasItems(p, moveTypes, slot, selected, shieldAt, bannerAt, toggleSlots) * 0.042f;
			      move(moveTypes, slot, moveBy);
			      float moveByArmor = hasEquipments(p, moveTypes, slot) ? 0.038f : 0;
		    	  move(moveTypes, slot, moveByArmor);

		    	  GlStateManager.func_179114_b(slotRotations[(slot * 3)] + add[3], 1, 0, 0);
		    	  GlStateManager.func_179114_b(slotRotations[(slot * 3) + 1] + add[4], 0, 1, 0);
			      GlStateManager.func_179114_b(slotRotations[(slot * 3) + 2] + add[5], 0, 0, 1);

			      TransformType ctx = TransformType.THIRD_PERSON_RIGHT_HAND;
//			      boolean rightHanded = p_174518_.getPrimaryHand() == HumanoidArm.RIGHT;
//			      if(slot == 12) {
//			    	  ctx = !rightHanded ? TransformType.THIRD_PERSON_RIGHT_HAND : TransformType.THIRD_PERSON_LEFT_HAND;
//			      }
			      
			      float scalePlus = (scale[slot]+0.001f + add[6])/100f;
		    	  float f = 1f;
		          GlStateManager.func_179152_a(f + scalePlus, f + scalePlus, f + scalePlus);

		    	  if(items[itemslot].func_77973_b() instanceof ItemBow) {
		    		  renderBowItem(itemslot, p_174518_, items[itemslot], ctx);
		    	  }else if((items[itemslot].func_77973_b() instanceof ItemTool || items[itemslot].func_77973_b() instanceof ItemSword) && !(items[itemslot].func_77973_b() instanceof ItemArmor)) {
		    		  renderTieredItem(itemslot, p_174518_, items[itemslot], ctx);
		      	  }else if(whiteListed(items[itemslot])) {
		    		  GlStateManager.func_179137_b(0.0F, 0.7D, 0.0D);
		    		  GlStateManager.func_179114_b(180, 0, 0, 1);
		    		  renderNormalItem(itemslot, p_174518_, items[itemslot], ctx);
		    	  }
		    	  GlStateManager.func_179121_F();
	    	  }
	      }
	      
	      GlStateManager.func_179121_F();
   }
   
   public void translateToHand(ModelRenderer part, EnumHandSide p_102854_) {
	   part.func_78794_c(0.0625F);
   }

//   public void translateRotate(MatrixStack p_104300_, float x, float y, float z, float xRot, float yRot, float zRot, float xScale, float yScale, float zScale) {
//	      p_104300_.translate((double)(x / 16.0F), (double)(y / 16.0F), (double)(z / 16.0F));
//	      if (zRot != 0.0F) {
//	         p_104300_.rotate(Vector3f.ZP.rotation(zRot));
//	      }
//
//	      if (yRot != 0.0F) {
//	         p_104300_.rotate(Vector3f.YP.rotation(yRot));
//	      }
//
//	      if (xRot != 0.0F) {
//	         p_104300_.rotate(Vector3f.XP.rotation(xRot));
//	      }
//
//	      if (xScale != 1.0F || yScale != 1.0F || zScale != 1.0F) {
//	         p_104300_.scale(xScale, yScale, zScale);
//	      }
//   }

   public void copyModelPartRotation(int slot, String[][] bodyattachment, float multiplier, float maxRotateChanger) {
	   ModelRenderer part = ((ModelBiped)modelrenderer.func_177087_b()).field_78115_e;
 	  if(bodyattachment[slot][1].equals("head")) {
 		  part = ((ModelBiped)modelrenderer.func_177087_b()).field_78116_c;
 	  }else if(bodyattachment[slot][1].equals("body")) {
 		  part = ((ModelBiped)modelrenderer.func_177087_b()).field_78115_e;
 	  }else if(bodyattachment[slot][1].equals("rarm")) {
 		  part = ((ModelBiped)modelrenderer.func_177087_b()).field_178723_h;
 	  }else if(bodyattachment[slot][1].equals("larm")) {
 		  part = ((ModelBiped)modelrenderer.func_177087_b()).field_178724_i;
 	  }else if(bodyattachment[slot][1].equals("rleg")) {
 		  part = ((ModelBiped)modelrenderer.func_177087_b()).field_178721_j;
 	  }else if(bodyattachment[slot][1].equals("lleg")) {
 		  part = ((ModelBiped)modelrenderer.func_177087_b()).field_178722_k;
 	  }
 	  part.func_78794_c(0.0625F);
   }

   public static void renderTridentItem(int slot, EntityPlayer p_174518_, ItemStack itemStack, TransformType ctx) {
		GlStateManager.func_179094_E();
		GlStateManager.func_179114_b(140f, 0, 0, 1);
		GlStateManager.func_179137_b(0.7F, 0.9D, 0.61D);
		GlStateManager.func_179152_a(0.9f, 0.9f, 0.9f);

		GlStateManager.func_179109_b(-0.45f, -1.07f, -0.535f);
		GlStateManager.func_179114_b(-60, 0, 1, 0);
		
        Minecraft.func_71410_x().func_175597_ag().func_187462_a(p_174518_, itemStack, ctx, false);
        GlStateManager.func_179121_F();
	}

	public static void renderCrossbowItem(int slot, EntityPlayer p_174518_, ItemStack itemStack, TransformType ctx) {
		GlStateManager.func_179094_E();
		GlStateManager.func_179137_b(-0.05D, 0.35D, 0.16D);
		GlStateManager.func_179114_b(180.0F, 0, 1, 0);
		GlStateManager.func_179114_b(270.0F, 0, 0, 1);
		GlStateManager.func_179152_a(0.8F, -0.8F, -0.8F);

		GlStateManager.func_179109_b(0.1f, -0.2f, 0);
//		GlStateManager.rotate(0, 0, 1, 0);
		GlStateManager.func_179114_b(60F, 0, 0, 1);
		GlStateManager.func_179114_b(90, 1, 0, 0);
		Minecraft.func_71410_x().func_175597_ag().func_187462_a(p_174518_, itemStack, ctx, false);
	       
		GlStateManager.func_179121_F();
	}
	
	public static void renderBowItem(int slot, EntityPlayer p_174518_, ItemStack itemStack, TransformType ctx) {
		GlStateManager.func_179094_E();
		GlStateManager.func_179137_b(0.0D, 0.35D, 0.16D);
		GlStateManager.func_179114_b(180.0F, 0, 1, 0);
		GlStateManager.func_179114_b(180.0F, 0, 0, 1);
		GlStateManager.func_179152_a(0.8F, -0.8F, -0.8F);
		
		GlStateManager.func_179137_b(-0.05d, 0.2d, -0.07D);
		GlStateManager.func_179114_b(97.0F, 0, 1, 0);
	 	GlStateManager.func_179114_b(-6.0F, 0, 0, 1);
	 	GlStateManager.func_179114_b(40.0F, 1, 0, 0);
	    
	 	Minecraft.func_71410_x().func_175597_ag().func_187462_a(p_174518_, itemStack, ctx, false);
	       
		GlStateManager.func_179121_F();
	}
	
	public static void renderNormalItem(int slot, EntityPlayer p_174518_, ItemStack itemStack, TransformType ctx) {
		GlStateManager.func_179094_E();
		GlStateManager.func_179137_b(0.0D, 0.35D, 0.16D);
		GlStateManager.func_179114_b(180.0F, 0, 1, 0);
		GlStateManager.func_179114_b(180.0F, 0, 0, 1);
		GlStateManager.func_179152_a(0.8F, -0.8F, -0.8F);

		GlStateManager.func_179109_b(0.1f, -0.1f, -0.08f);
		Minecraft.func_71410_x().func_175597_ag().func_187462_a(p_174518_, itemStack, ctx, false);
	       
		GlStateManager.func_179121_F();
	}
	
	public static void renderTieredItem(int slot, EntityPlayer p_174518_, ItemStack itemStack, TransformType ctx) {
		GlStateManager.func_179094_E();
	  	GlStateManager.func_179137_b(-0.05D, 0.3D, 0.16D);
	  	GlStateManager.func_179114_b(180F, 0, 1, 0);
	 	GlStateManager.func_179114_b(180, 0, 0, 1);
		GlStateManager.func_179152_a(0.8F, -0.8F, -0.8F);
		GlStateManager.func_179137_b(-0.2D, -0.10D, -0.0D);
		GlStateManager.func_179114_b(90.0F, 0, 1, 0);
		GlStateManager.func_179114_b(55.0F, 1, 0, 0);

		Minecraft.func_71410_x().func_175597_ag().func_187462_a(p_174518_, itemStack, ctx, false);
	    
		GlStateManager.func_179121_F();
	}
	
	public static int[] uniqueSettings(ItemStack stack, int slotNumber) {
		String selectedSlotNumber = "0";
		
		if(slotNumber < 9) {
			selectedSlotNumber = String.valueOf(slotNumber+1);
		}else if(slotNumber == 9) {
			selectedSlotNumber = "shield";
		}else if(slotNumber == 10) {
			selectedSlotNumber = "banner";
		}else if(slotNumber == 11) {
			selectedSlotNumber = "offhand";
		}else if(slotNumber == 12) {
			selectedSlotNumber = "quiver";
		}
		
		if(stack != null) {
			if(stack.func_77973_b() != null) {
				if(ModUtils.stringContains(ClientOnlyForgeSetup.uniqueItemDisplay, ",")) {
					if(ModUtils.stringContains(ClientOnlyForgeSetup.uniqueItemDisplay, " ")) { //list
						String[] allData = ClientOnlyForgeSetup.uniqueItemDisplay.split(" ");
						
						for(int i = 0; i < allData.length; i++) {
							if(ModUtils.stringContains(allData[i], ",")) {
								String name = StringUtils.substringBefore(allData[i], ",");
								String[] data = allData[i].split(",");
								
								if(ModUtils.arrayExist(data, 9)) {
									String itemname = "";
									if(stack.func_77973_b() != null && ModUtils.stringExist(stack.func_77973_b().getRegistryName().func_110623_a())) {
										itemname = stack.func_77973_b().getRegistryName().func_110623_a();
										
										if(ModUtils.stringContains(itemname, ".")) {
											itemname = StringUtils.substringAfterLast(itemname, ".");
										}
									}
									
									if(ModUtils.stringExist(itemname) && itemname.equals(name)
											&& ModUtils.stringEquals(data[8], selectedSlotNumber)) {
										return getData(allData[i]);
									}
								}
							}
						}
						
					}else {
						String name = StringUtils.substringBefore(ClientOnlyForgeSetup.uniqueItemDisplay, ",");
						String[] data = ClientOnlyForgeSetup.uniqueItemDisplay.split(",");
						
						if(ModUtils.arrayExist(data, 9)) {
							String itemname = "";
							if(stack.func_77973_b() != null && ModUtils.stringExist(stack.func_77973_b().getRegistryName().func_110623_a())) {
								itemname = stack.func_77973_b().getRegistryName().func_110623_a();
								
								if(ModUtils.stringContains(itemname, ".")) {
									itemname = StringUtils.substringAfterLast(itemname, ".");
								}
							}
							
							if(ModUtils.stringExist(itemname) && itemname.equals(name)
									&& ModUtils.stringEquals(data[8], selectedSlotNumber)) {
								return getData(ClientOnlyForgeSetup.uniqueItemDisplay);
							}
						}
					}
				}
			}
		}
		
		return new int[] {0, 0, 0, 0, 0, 0, 0};
	}
	
	public static int[] getData(String s) {
		String[] txtArray = s.split(",");
		
		if(ModUtils.arrayExist(txtArray, 8)) {
			int x = 0;
			if(NumberUtils.isNumber(txtArray[1])) {
				x = NumberUtils.createInteger(txtArray[1]);
			}
			int y = 0;
			if(NumberUtils.isNumber(txtArray[2])) {
				y = NumberUtils.createInteger(txtArray[2]);
			}
			int z = 0;
			if(NumberUtils.isNumber(txtArray[3])) {
				z = NumberUtils.createInteger(txtArray[3]);
			}
			int xrot = 0;
			if(NumberUtils.isNumber(txtArray[4])) {
				xrot = NumberUtils.createInteger(txtArray[4]);
			}
			int yrot = 0;
			if(NumberUtils.isNumber(txtArray[5])) {
				yrot = NumberUtils.createInteger(txtArray[5]);
			}
			int zrot = 0;
			if(NumberUtils.isNumber(txtArray[6])) {
				zrot = NumberUtils.createInteger(txtArray[6]);
			}
			int scale = 0;
			if(NumberUtils.isNumber(txtArray[7])) {
				scale = NumberUtils.createInteger(txtArray[7]);
			}
			
			return new int[] {x, y, z, xrot, yrot, zrot, scale};
		}
		
		return new int[] {0, 0, 0, 0, 0, 0, 0};
	}

	@Override
	public boolean func_177142_b() {
		return false;
//		return true;
	}
}
